import { __awaiter } from 'tslib';
import { ɵɵdefineInjectable, ɵɵinject, Injectable } from '@angular/core';
import { DcSccBaseFactory, DcSccBaseValidators, DcScc } from 'dc-scc';
import { DcEasApi } from 'dc-eas-api';
import { DcDdsObjectModelV1 } from 'dc-dds-api';

class BaseFactory extends DcSccBaseFactory {
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.getService().newRequest();
            return reqObj;
        });
    }
    clearCache(obj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[obj.getGuid()] != "undefined") {
                delete this._s[obj.getGuid()];
            }
            return this;
        });
    }
    getByGuids(guids, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var utilObj = this.getService().getDcScc().getUtilities().getFactories().getAsyncGuids();
            return yield utilObj.getByGuids(guids, this, "getByGuid", cbObj, cbMethod);
        });
    }
}

class Connection extends BaseFactory {
    constructor() {
        super(...arguments);
        this._wsHost = "dc-wss-nlb.datacamo.net";
        this._wsPort = 28109;
        this._wsSsl = true;
        this._isReconn = false;
        this._wrkObj = null;
    }
    setConfig(host, port, ssl = true) {
        this._wsHost = host;
        this._wsPort = port;
        this._wsSsl = ssl;
        return this;
    }
    getClient() {
        return __awaiter(this, void 0, void 0, function* () {
            var wrkObj = yield this.getWorker();
            return wrkObj.getClient();
        });
    }
    getWorker() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._wrkObj === null) {
                try {
                    this._wrkObj = this.getService().getDcScc().getWorkers().getV3(this._wsHost, this._wsPort, this._wsSsl);
                    yield this._wrkObj.initialize();
                    var cObj = this._wrkObj.getClient();
                    var reqObj = cObj.newMessage();
                    reqObj.setL1("Authentication").setL2("Get").setL3("Salt");
                    var salt = yield reqObj.getResponse(true);
                    var reqObj = cObj.newMessage();
                    reqObj.setL1("Authentication").setL2("Validate").setL3("ByEAS").setL4("EphemeralKey");
                    var hash = yield this.getMtm().get("utilities").getHashing().getSHA().get256("UNR--" + salt);
                    var ephKey = yield this.getMtm().get("DcEasApi").getEphemeralKeys().newReadUserPublic();
                    yield ephKey.signRequest(reqObj, hash);
                    var sessObj = yield reqObj.getResponse(true);
                    if (typeof sessObj["session-id"] != "undefined"
                        && typeof sessObj["session-key"] != "undefined") {
                        cObj.setSession(sessObj["session-id"], sessObj["session-key"]);
                        this._wrkObj.setTermCb(this, "reconnect");
                    }
                    else {
                        throw this.getMtm().newException("Server authentication returned an invalid session");
                    }
                }
                catch (errObj) {
                    this._wrkObj = null;
                    throw errObj;
                }
            }
            return this._wrkObj;
        });
    }
    reconnect(wrkObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._isReconn === false) {
                this._isReconn = true;
                this.logInfo("UNR Reconnecting");
                try {
                    try {
                        this._wrkObj = null;
                        yield this.getWorker();
                        this._isReconn = false;
                        this.logInfo("UNR Reconnect success");
                    }
                    catch (errObj) {
                        throw this.getService().getMtm().newException("UNR Reconnect failed");
                    }
                }
                catch (errObj) {
                    this._isReconn = false;
                    this.callException(errObj);
                }
            }
        });
    }
}

class Base extends DcDdsObjectModelV1 {
    constructor() {
        super(...arguments);
        this._guid = null;
    }
    baseConstruct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            this.setExceptionCb(this.getService().getLogging(), "exception");
            this._guid = apiObj.guid;
            if (typeof apiObj["ddsId"] != "undefined") {
                var ddsObj = yield this.getService().getDdsClient();
                this.setConfig(ddsObj, apiObj.ddsId, apiObj.ddsAuth);
                var storeObj = yield this.pubStore();
                yield storeObj.initialize();
            }
        });
    }
    getGuid() {
        return this._guid;
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.getService().newRequest();
            return reqObj;
        });
    }
}

class User extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getDisplayName() {
        return this.getPubKey("displayName", true);
    }
    getRootMaps(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getRootMaps().getByUser(this, cbObj, cbMethod);
        });
    }
    getNsMaps(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNsMaps().getByUser(this, cbObj, cbMethod);
        });
    }
    getNsRoleMaps(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNsRoleMaps().getByUser(this, cbObj, cbMethod);
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getUsers().clearCache(this);
    }
}

class Users extends BaseFactory {
    newObj() {
        var rObj = new User();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getBySession(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            //get all users this user has access to
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("BySession");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid]);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
    addByEas(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var reqObj = yield this.newRequest();
                reqObj.setL1("Users").setL2("Add").setL3("ByEAS");
                var hash = yield this.getMtm().get("utilities").getHashing().getSHA().get256("USM--ADD-ME");
                var ephKey = yield this.getMtm().get("DcEasApi").getEphemeralKeys().newReadUserPublic();
                yield ephKey.signRequest(reqObj, hash);
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class Session extends DcSccBaseValidators {
    constructor() {
        super(...arguments);
        this._cObj = null;
    }
    construct(cObj) {
        return __awaiter(this, void 0, void 0, function* () {
            this._cObj = cObj;
            this.setInit();
            return this;
        });
    }
    getRootSpaces(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getRootSpaces().getBySession(cbObj, cbMethod);
        });
    }
    getNameSpaces(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNameSpaces().getBySession(cbObj, cbMethod);
        });
    }
    getUsers(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUsers().getBySession(cbObj, cbMethod);
        });
    }
    getNsRoles(cbObj = null, cbMethod = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNsRoles().getBySession(cbObj, cbMethod);
        });
    }
}

class DcUnrApi {
    constructor(_dcScc, _dcEasApi) {
        this._dcScc = _dcScc;
        this._dcEasApi = _dcEasApi;
        this._connFact = null;
        this._sessObj = null;
        this._uFact = null;
        this._ddsClient = null;
        this._dcScc.getMtm().set("DcUnrApi", this);
    }
    getDcScc() {
        return this._dcScc;
    }
    getMtm() {
        return this.getDcScc().getMtm();
    }
    getLogging() {
        return this.getDcScc().getLogging();
    }
    getSession() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._sessObj === null) {
                this._sessObj = new Session();
                this._sessObj.setService(this);
                this._sessObj.construct(yield this.getConnection().getClient());
                this._sessObj.setExceptionCb(this.getLogging(), "exception");
            }
            return this._sessObj.waitInit();
        });
    }
    getConnection() {
        if (this._connFact === null) {
            this._connFact = new Connection();
            this._connFact.setService(this);
            this._connFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._connFact;
    }
    getUsers() {
        if (this._uFact === null) {
            this._uFact = new Users();
            this._uFact.setService(this);
            this._uFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._uFact;
    }
    getDdsClient() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._ddsClient === null) {
                var ddsHost = "dc-wss-nlb.datacamo.net";
                var ddsPort = 28106;
                var ddsSsl = true;
                this._ddsClient = this.getMtm().get("DcDdsApi").getClients().getV1(ddsHost, ddsPort, ddsSsl);
                yield this._ddsClient.initialize();
            }
            return this._ddsClient;
        });
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var cObj = yield this.getConnection().getClient();
            var cObj = yield cObj.isAuth();
            return cObj.newMessage();
        });
    }
}
DcUnrApi.ɵprov = ɵɵdefineInjectable({ factory: function DcUnrApi_Factory() { return new DcUnrApi(ɵɵinject(DcScc), ɵɵinject(DcEasApi)); }, token: DcUnrApi, providedIn: "root" });
DcUnrApi.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
DcUnrApi.ctorParameters = () => [
    { type: DcScc },
    { type: DcEasApi }
];

/*
 * Public API Surface of dc-unr-api
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DcUnrApi };
//# sourceMappingURL=dc-unr-api.js.map
